"""
Django settings for zeppsta project.

Generated by 'django-admin startproject' using Django 1.11.16.

For more information on this file, see
https://docs.djangoproject.com/en/1.11/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.11/ref/settings/
"""

import getpass
import importlib
import locale
import logging
import os
from enum import Enum

from django.contrib.messages import constants as message_constants
from django.core.exceptions import ImproperlyConfigured
from django.urls import NoReverseMatch, reverse

logger = logging.getLogger(__name__)


class Environment(Enum):
    """Deployment environments supported by zeppsta"""
    LOCAL = 'local'
    STAGING = 'staging'
    PROD = 'prod'


# zeppsta-specific configuration
ACTIVE_ENV = Environment(os.environ.get('ZEPPSTA_ACTIVE_ENV', 'local'))
BASE_URL = os.environ.get('DJANGO_BASE_URL', 'https://localhost:8000/')
MAIN_APP = os.environ['DJANGO_SETTINGS_MODULE'].split('.')[0]


# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.environ['DJANGO_BASE_DIR']
DATA_DIR = os.path.join(BASE_DIR, 'data')


# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.environ['DJANGO_SECRET_KEY']


# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = Environment.PROD is not ACTIVE_ENV

ALLOWED_HOSTS = os.environ.get('DJANGO_ALLOWED_HOSTS', 'localhost,localhost.localdomain')
if ALLOWED_HOSTS:
    ALLOWED_HOSTS = [host.strip() for host in ALLOWED_HOSTS.split(',')]


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'zeppsta',
]

if Environment.LOCAL is ACTIVE_ENV:
    INSTALLED_APPS.extend([
        'debug_toolbar',
        'django_extensions',
    ])

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'htmlmin.middleware.HtmlMinifyMiddleware',
    'htmlmin.middleware.MarkRequestMiddleware',
]

if Environment.LOCAL is ACTIVE_ENV:
    MIDDLEWARE.extend([
        'debug_toolbar.middleware.DebugToolbarMiddleware',
    ])

ROOT_URLCONF = '{}.urls'.format(MAIN_APP)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates', 'django')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
    {
        'BACKEND': 'zeppsta.jinja2.Jinja2',
        'DIRS': [os.path.join(BASE_DIR, 'templates', 'jinja2')],
        'APP_DIRS': True,
        'OPTIONS': {
            'environment': 'zeppsta.jinja2.environment'
        },
    },
]


# Database
# https://docs.djangoproject.com/en/1.11/ref/settings/#databases

DATABASES = {
    'local': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': os.environ.get('DJANGO_DB_NAME', MAIN_APP),
        'USER': os.environ.get('DJANGO_DB_USER', getpass.getuser()),
        'PASSWORD': os.environ.get('DJANGO_DB_PASSWORD', ''),
        'HOST': os.environ.get('DJANGO_DB_HOST', 'localhost'),
        'PORT': os.environ.get('DJANGO_DB_PORT', 5432),
    },
    'remote': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': os.environ.get('DJANGO_DB_NAME'),
        'USER': os.environ.get('DJANGO_DB_USER'),
        'PASSWORD': os.environ.get('DJANGO_DB_PASSWORD'),
        'HOST': os.environ.get('DJANGO_DB_HOST'),
        'PORT': os.environ.get('DJANGO_DB_PORT', 5432),
    },
}

DATABASES['default'] = DATABASES['local'] if Environment.LOCAL is ACTIVE_ENV else DATABASES['remote']


# Password validation
# https://docs.djangoproject.com/en/1.11/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

if not MAIN_APP == 'zeppsta':
    AUTH_USER_MODEL = '{}.User'.format(MAIN_APP)


# Internationalization
# https://docs.djangoproject.com/en/1.11/topics/i18n/

LANGUAGE_CODE = os.environ.get('LANG', 'en_US.utf8')
try:
    locale.setlocale(locale.LC_ALL, LANGUAGE_CODE)
except locale.Error as e:
    logger.warning(str(e))

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.11/howto/static-files/

STATIC_ROOT = os.path.join(BASE_DIR, 'static', '_root')

STATIC_URL = os.environ.get('DJANGO_STATIC_URL', '/static/')

STATICFILES_DIRS = ['public']


MEDIA_URL = os.environ.get('DJANGO_MEDIA_URL', '/media/')
if Environment.LOCAL is ACTIVE_ENV:
    DEFAULT_FILE_STORAGE = 'django.core.files.storage.FileSystemStorage'
    MEDIA_ROOT = os.path.join(BASE_DIR, 'media')
else:
    DEFAULT_FILE_STORAGE = 'storages.backends.sftpstorage.SFTPStorage'
    SFTP_STORAGE_HOST = os.environ.get('DJANGO_SFTP_STORAGE_HOST')
    SFTP_STORAGE_ROOT = os.environ.get('DJANGO_SFTP_STORAGE_ROOT')
    SFTP_STORAGE_PARAMS = {
        'username': os.environ.get('DJANGO_SFTP_STORAGE_USER'),
        'password': os.environ.get('DJANGO_SFTP_STORAGE_PASSWORD'),
    }


# Email configuration

if Environment.LOCAL is ACTIVE_ENV:
    EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'
else:
    EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'
    EMAIL_HOST = os.environ.get('DJANGO_EMAIL_HOST')
    EMAIL_PORT = os.environ.get('DJANGO_EMAIL_PORT', 587)
    EMAIL_USE_TLS = True
    DEFAULT_FROM_EMAIL = os.environ.get('DJANGO_DEFAULT_FROM_EMAIL')
    EMAIL_HOST_USER = os.environ.get('DJANGO_EMAIL_HOST_USER')
    EMAIL_HOST_PASSWORD = os.environ.get('DJANGO_EMAIL_HOST_PASSWORD')


LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
        },
    },
    'loggers': {
        '': {
            'handlers': ['console'],
            'level': os.getenv('DJANGO_LOG_LEVEL', 'DEBUG'),
        },
    },
}


MESSAGE_LEVEL = message_constants.DEBUG if DEBUG else message_constants.INFO


THUMBNAIL_DEFAULT_GEOMETRY_STRING = "500x500"
THUMBNAIL_PRESERVE_FORMAT = True


INTERNAL_IPS = ['127.0.0.1']


CLIENT_CONTEXT_SETTINGS = ['MEDIA_URL', 'STATIC_URL']


try:
    import django_registration
    ACCOUNT_ACTIVATION_DAYS = 3
except ImportError:
    pass
